// Ampelsteuerung

// Ampelsteuerung an einer einfachen Kreuzung
// Die Ampelphasen werden über eine Tabelle definiert

#include <MobaTools.h>

// Definition der Ausgangspins:
const byte hRotPin   = A0;  // Ampel an der Hauptstraße
const byte hGelbPin  = A1;
const byte hGruenPin = A2;
const byte nRotPin   = A3;  // Ampel an der Nebenstraße
const byte nGelbPin  = A4;
const byte nGruenPin = A5;


// Tabelle für die Ampelzustände
// Ampelphase:                1  | 2  | 3  | 4  | 5  | 6  | 7  | 8
// Tabellenindex:             0  | 1  | 2  | 3  | 4  | 5  | 6  | 7
const byte zeitDauer  [] = { 30  , 3  , 3  , 3  , 20 , 3  , 3  , 3  };
const bool hauptRot   [] = { OFF ,OFF , ON , ON , ON , ON , ON , ON };
const bool hauptGelb  [] = { OFF , ON ,OFF ,OFF ,OFF ,OFF ,OFF , ON };
const bool hauptGruen [] = {  ON ,OFF ,OFF ,OFF ,OFF ,OFF ,OFF ,OFF };
const bool nebenRot   [] = {  ON , ON , ON , ON ,OFF ,OFF , ON , ON };
const bool nebenGelb  [] = { OFF ,OFF ,OFF , ON ,OFF , ON ,OFF ,OFF };
const bool nebenGruen [] = { OFF ,OFF ,OFF ,OFF , ON ,OFF ,OFF ,OFF };

const byte tabMax = sizeof(zeitDauer)-1; // Tabellenindex startet bei 0 ( deshalb die -1 für den höchsten Index )
byte ampelTabIx;   // läuft von 0 .. 7

// Einrichten der notwendigen MoBaTools Objekte:
MoToTimer AmpelTimer;
MoToSoftLed hRot;
MoToSoftLed hGelb;
MoToSoftLed hGruen;
MoToSoftLed nRot;
MoToSoftLed nGelb;
MoToSoftLed nGruen;


void setup() {
    // Softled Objekte initiieren
    hRot.attach( hRotPin );
    hRot.riseTime( 200 );
    hGelb.attach( hGelbPin );
    hGelb.riseTime( 200 );
    hGruen.attach( hGruenPin );
    hGruen.riseTime( 200 );
    nRot.attach( nRotPin );
    nRot.riseTime( 200 );
    nGelb.attach( nGelbPin );
    nGelb.riseTime( 200 );
    nGruen.attach( nGruenPin );
    nGruen.riseTime( 200 );

    ampelTabIx = 0;    // Das System startet mit freier Fahrt auf der Hauptstraße
    
}

void loop() {
    // Tabellengesteuerte Ampel
    if( !AmpelTimer.running() ) {
        // Zeit abgelaufen
        // Ampel schalten
        hRot.write( hauptRot[ampelTabIx] );
        hGelb.write( hauptGelb[ampelTabIx] );
        hGruen.write( hauptGruen[ampelTabIx] );
        nRot.write( nebenRot[ampelTabIx] );
        nGelb.write( nebenGelb[ampelTabIx] );
        nGruen.write( nebenGruen[ampelTabIx] );
        // Zeit setzen ( da der Timer in ms rechnet, müssen die Tabellenwerte mit 1000
        // multipliziert werden )
        AmpelTimer.setTime( zeitDauer[ampelTabIx] * 1000 );
        // Merker weiterrückem
        if( ampelTabIx >= tabMax ) {
            // Tabellenende erreicht
            ampelTabIx = 0;
        } else {
            ampelTabIx++;
        }
    }
}
