// Wechselblinker + Ampelsteuerung

#include <MobaTools.h>

//vvvvvvvvvvvvvvvvvvvvvvvvv BEGIN WECHSELBLINKER vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
// Definitionen für den Wechselblinker
// Festlegen der Anschlüsse (Pinnummer) für Ein- und Ausgänge
const byte linksPin = 4;     // linke Lampe
const byte rechtsPin = 5;    // rechte Lampe
const byte belegtPin = 8;    // Eingang 'Zug kommt'

// Benötigte MobaTools Objekte:
MoToTimer StoppUhr; // Zum Messen von Zeiten
MoToSoftLed LampeLinks;
MoToSoftLed LampeRechts;

// Zeiten für den Blinkrhythmus ( in ms )
const int STARTZEIT = 400;
const int BLINKZEIT = 500;
const int FADEZEIT = 450;

EggTimer EntPrellung; // Zur Tasterentporellung
const int PRELLZEIT = 100;

// Variable
boolean bueBelegt; // = TRUE, wenn sich ein Zug im Bereich des Bue befindet
byte    wBlinkerZustand;    // aktueller Zustand unseres Blinkerautomaten
//^^^^^^^^^^^^^^^^^^^^^^^^ ENDE WECHSELBLINKER ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    
//vvvvvvvvvvvvvvvvvvvvvvvv BEGIN AMPELSTEUERUNG vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
// Ampelsteuerung an einer einfachen Kreuzung
// Die Ampelphasen werden über eine Tabelle definiert

// Definitionen für die Ampelschaltung:
// Definition der Ausgangspins:
const byte hRotPin   = A0;  // Ampel an der Hauptstraße
const byte hGelbPin  = A1;
const byte hGruenPin = A2;
const byte nRotPin   = A3;  // Ampel an der Nebenstraße
const byte nGelbPin  = A4;
const byte nGruenPin = A5;


// Tabelle für die Ampelzustände
// Ampelphase:                1  | 2  | 3  | 4  | 5  | 6  | 7  | 8
// Tabellenindex:             0  | 1  | 2  | 3  | 4  | 5  | 6  | 7
const byte zeitDauer  [] = { 30  , 3  , 3  , 3  , 20 , 3  , 3  , 3  };
const bool hauptRot   [] = { OFF ,OFF , ON , ON , ON , ON , ON , ON };
const bool hauptGelb  [] = { OFF , ON ,OFF ,OFF ,OFF ,OFF ,OFF , ON };
const bool hauptGruen [] = {  ON ,OFF ,OFF ,OFF ,OFF ,OFF ,OFF ,OFF };
const bool nebenRot   [] = {  ON , ON , ON , ON ,OFF ,OFF , ON , ON };
const bool nebenGelb  [] = { OFF ,OFF ,OFF , ON ,OFF , ON ,OFF ,OFF };
const bool nebenGruen [] = { OFF ,OFF ,OFF ,OFF , ON ,OFF ,OFF ,OFF };

const byte tabMax = sizeof(zeitDauer)-1; // Tabellenindex startet bei 0 ( deshalb die -1 für den höchsten Index )
byte ampelTabIx;   // läuft von 0 .. 7

// Einrichten der notwendigen MoBaTools Objekte:
MoToTimer AmpelTimer;
MoToSoftLed hRot;
MoToSoftLed hGelb;
MoToSoftLed hGruen;
MoToSoftLed nRot;
MoToSoftLed nGelb;
MoToSoftLed nGruen;
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^ ENDE AMPELSTEUERUNG ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//----------------------------------------------------------------------

void setup() {
    //vvvvvvvvvvvvvvvvvvvvvvvvv BEGIN WECHSELBLINKER vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    // Initiierungen Wechselblinker
    // Pins einrichten
    pinMode( belegtPin, INPUT_PULLUP ); // Mit einem Schalter nach GND wird Blinker eingeschaltet
    LampeLinks.attach( linksPin );      // Zuordnung der Softleds zu den Ausgangspins
    LampeRechts.attach( rechtsPin );
    LampeLinks.riseTime( FADEZEIT );    // Überblendzeit einstellen
    LampeRechts.riseTime( FADEZEIT );

    wBlinkerZustand = 1;        // Startzustand des Blinkautomaten.
    //^^^^^^^^^^^^^^^^^^^^^^^^ ENDE WECHSELBLINKER ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    
    //vvvvvvvvvvvvvvvvvvvvvvvv BEGIN AMPELSTEUERUNG vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    //Initiierungen für die Ampel    
    // Softled Objekte initiieren
    hRot.attach( hRotPin );
    hRot.riseTime( 200 );
    hGelb.attach( hGelbPin );
    hGelb.riseTime( 200 );
    hGruen.attach( hGruenPin );
    hGruen.riseTime( 200 );
    nRot.attach( nRotPin );
    nRot.riseTime( 200 );
    nGelb.attach( nGelbPin );
    nGelb.riseTime( 200 );
    nGruen.attach( nGruenPin );
    nGruen.riseTime( 200 );

    ampelTabIx = 0;    // Das System startet mit freier Fahrt auf der Hauptstraße
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^ ENDE AMPELSTEUERUNG ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//----------------------------------------------------------------------
}

void loop() {
    
    //vvvvvvvvvvvvvvvvvvvvvvvvv BEGIN WECHSELBLINKER vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    // Wechselblinkerautomat
    // Belegtzusatnd abfragen. Das machen wir hier jetzterstmal ganz einfach: Letztendlich
    // schalten wir mit dem Eingang einfach den Wechselblinker ein und aus.
    // Im realen Praxisfall könnte hier gegebenenfalls auch ein komplexere Logik stehen, um zu
    // erkennen, ob ein Zug sich nähert oder den Bü wieder verlassen hat.
    //if ( not EntPrellung.running() ) {
    //    EntPrellung.setTime( PRELLZEIT );
        bueBelegt = not digitalRead( belegtPin ); // ein LOW-Level am Eingangspin schaltet den Blinker ein
    //}
    
    switch ( wBlinkerZustand ) {
      // je nach Wert der Variable wBlinkerZustand, wird die entsprechende 'case' Zeile angesprungen
      case 1: // Grundzustand, abfragen, ob sich ein Zug nähert ( z.B. über einen Belegtmelder )
        if ( bueBelegt ) {
          // Der Zug ist in den Belegtabschnitt vor der Schranke eingefahren
          // beide Lampen einschalten
          LampeLinks.on();
          LampeRechts.on();
          // Stoppuhr mit der Zeit die beide Lampen an sein sollen vorbelegen
          StoppUhr.setTime( STARTZEIT ); // StoppUhr 'aufziehen'
          // in den 2. Zustand wechseln
          wBlinkerZustand = 2; // Ab dem nächsten loop-Durchlauf werden nur die Anweisungen des case 2 ausgeführt
        }
        break; // Ende der Anweisungen für den 1. Zustand
      //-------------------------------------------------------------------------
      case 2: // beide Lampen ein, wir fragen ab, ob die Stoppuhr abgelaufen ist.
        if ( not StoppUhr.running() ) {
          // Die Stoppuhr läuft nicht mehr, die Zeit ist um
          LampeRechts.off();
          StoppUhr.setTime( BLINKZEIT ); // Stoppuhr 'aufziehen'
          // in den 3. Zustand wechseln
          wBlinkerZustand = 3; // Ab dem nächsten loop-Durchlauf werden nur die Anweisungen des case 3 ausgeführt
        }
        break; // Ende der Anweisungen für den 2. Zustand
      //-------------------------------------------------------------------------
      case 3: // linke Lampen ist an, rechte aus: wir fragen ab, ob die Stoppuhr abgelaufen ist.
        if ( not StoppUhr.running() ) {
          // Die Stoppuhr läuft nicht mehr, die Zeit ist um, Lampen umschalten
          LampeLinks.off();
          // Im 3. Zustand müssen wir auch noch abfragen, ob der Zug den Belegtabschnitt verlassen hat
            if ( not bueBelegt ) {
                // Der Zug ist komplett über den Bü gefahren, die Straße ist wieder frei
                // Beide Lampen sind aus!
                // wieder in den 1. Zustand wechseln. Eine Zeitüberwachung wird nicht gebraucht
                wBlinkerZustand = 1; // Ab dem nächsten loop-Durchlauf werden nur die Anweisungen des case 1 ausgeführt
            }  else {
                LampeRechts.on();
                StoppUhr.setTime( BLINKZEIT ); // Stoppuhr 'aufziehen'
                // wieder in den 4. Zustand wechseln
                wBlinkerZustand = 4; // Ab dem nächsten loop-Durchlauf werden nur die Anweisungen des case 3 ausgeführt
            }
        }
        break; // Ende der Anweisungen für den 3. Zustand
      //-------------------------------------------------------------------------
      case 4: // linke Lampen ist aus, rechte an: wir fragen ab, ob die Stoppuhr abgelaufen ist.
        if ( not StoppUhr.running() ) {
          // Die Stoppuhr läuft nicht mehr, die Zeit ist um, Lampen umschalten
          LampeRechts.off();
          // Im 4. Zustand müssen wir auch noch abfragen, ob der Zug den Belegtabschnitt verlassen hat
            if ( not bueBelegt ) {
                // Der Zug ist komplett über den Bü gefahren, die Straße ist wieder frei
                // Beide Lampen sind aus!
                // wieder in den 1. Zustand wechseln. Eine Zeitüberwachung wird nicht gebraucht
                wBlinkerZustand = 1; // Ab dem nächsten loop-Durchlauf werden nur die Anweisungen des case 1 ausgeführt
            }  else {
                LampeLinks.on();
                StoppUhr.setTime( BLINKZEIT ); // Stoppuhr 'aufziehen'
                // wieder in den 3. Zustand wechseln
                wBlinkerZustand = 3; // Ab dem nächsten loop-Durchlauf werden nur die Anweisungen des case 3 ausgeführt
            }
        }
        break; // Ende der Anweisungen für den 4. Zustand
    } // Ende des switch-Blockes   
    //^^^^^^^^^^^^^^^^^^^^^^^^ ENDE WECHSELBLINKER ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    
    //vvvvvvvvvvvvvvvvvvvvvvvv BEGIN AMPELSTEUERUNG vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    // Tabellengesteuerte Ampel
    if( !AmpelTimer.running() ) {
        // Zeit abgelaufen
        // Ampel schalten
        hRot.write( hauptRot[ampelTabIx] );
        hGelb.write( hauptGelb[ampelTabIx] );
        hGruen.write( hauptGruen[ampelTabIx] );
        nRot.write( nebenRot[ampelTabIx] );
        nGelb.write( nebenGelb[ampelTabIx] );
        nGruen.write( nebenGruen[ampelTabIx] );
        // Zeit setzen ( da der Timer in ms rechnet, müssen die Tabellenwerte mit 1000
        // multipliziert werden )
        AmpelTimer.setTime( zeitDauer[ampelTabIx] * 1000 );
        // Merker weiterrückem
        if( ampelTabIx >= tabMax ) {
            // Tabellenende erreicht
            ampelTabIx = 0;
        } else {
            ampelTabIx++;
        }
    }
    //^^^^^^^^^^^^^^^^^^^^^^^^^^^^ ENDE AMPELSTEUERUNG ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
}
